import { Echo } from "@atums/echo";
import { redis } from "bun";
import { verifyRequiredVariables } from "#environment/config";
import { cassandra } from "#lib/database";

const echo = new Echo({
	disableFile: true,
});

async function resetCassandra(): Promise<void> {
	echo.info("Resetting Cassandra database...");

	try {
		verifyRequiredVariables();

		await cassandra.connect({ withKeyspace: false, logging: true });

		await cassandra.dropEverything();

		echo.info("Cassandra database reset complete");
		echo.info(
			"Restart your server to recreate the database and run migrations",
		);
	} catch (error) {
		echo.error({ message: "Failed to reset Cassandra:", error });
		process.exit(1);
	}
}

async function resetRedis(): Promise<void> {
	echo.info("Resetting Redis database...");
	try {
		verifyRequiredVariables();

		const keys = await redis.keys("*");

		if (keys.length > 0) {
			echo.info(`Found ${keys.length} keys to delete`);

			let deletedCount = 0;
			for (const key of keys) {
				await redis.del(key);
				deletedCount++;

				if (deletedCount % 100 === 0) {
					echo.info(`Deleted ${deletedCount}/${keys.length} keys...`);
				}
			}

			echo.info(`Deleted ${deletedCount} keys`);
		} else {
			echo.info("No keys found - Redis is already empty");
		}

		echo.info("Redis database reset complete");
		echo.info("All Redis data has been cleared");
	} catch (error) {
		echo.error({ message: "Failed to reset Redis:", error });
		process.exit(1);
	}
}

async function resetAll(): Promise<void> {
	echo.info("Resetting all databases...");

	try {
		await resetCassandra();
		await resetRedis();

		echo.info("All databases reset complete");
	} catch (error) {
		echo.error({ message: "Failed to reset databases:", error });
		process.exit(1);
	}
}

function showHelp(): void {
	echo.info("Available commands:");
	echo.info("  --reset cassandra    Reset Cassandra database (drops keyspace)");
	echo.info("  --reset redis        Reset Redis database (flush all data)");
	echo.info("  --reset all          Reset both databases");
	echo.info("  --help              Show this help message");
	echo.info("");
	echo.info("Examples:");
	echo.info("  bun run command --reset cassandra");
	echo.info("  bun run command --reset redis");
	echo.info("  bun run command --reset all");
}

export async function handleCommands(): Promise<boolean> {
	const args = process.argv.slice(2);

	const resetIndex = args.indexOf("--reset");
	if (resetIndex !== -1) {
		const resetTarget = args[resetIndex + 1];

		switch (resetTarget) {
			case "cassandra":
				await resetCassandra();
				return true;
			case "redis":
				await resetRedis();
				return true;
			case "all":
				await resetAll();
				return true;
			default:
				echo.error(`Unknown reset target: ${resetTarget}`);
				showHelp();
				process.exit(1);
		}
	}

	if (args.includes("--help") || args.includes("-h")) {
		showHelp();
		return true;
	}

	return false;
}
