import { echo } from "@atums/echo";
import { validateJWTConfig, validateMailerConfig } from "#lib/validation";
import { isValidUrl } from "#lib/validation/url";
import { requiredVariables } from "./constants";
import { cassandraConfig, validateCassandraConfig } from "./database";
import { jwt } from "./jwt";
import { mailerConfig } from "./mailer";

import type { Environment } from "#types/config";

const environment: Environment = {
	port: Number.parseInt(process.env.PORT || "8080", 10),
	host: process.env.HOST || "0.0.0.0",
	development:
		process.env.NODE_ENV === "development" || process.argv.includes("--dev"),
	frontendFqdn: process.env.FRONTEND_FQDN || "",
	backendFqdn: process.env.BACKEND_FQDN || "",
};

function verifyRequiredVariables(): void {
	let hasError = false;

	for (const key of requiredVariables) {
		const value = process.env[key];
		if (value === undefined || value.trim() === "") {
			echo.error(`Missing or empty environment variable: ${key}`);
			hasError = true;
		}
	}

	const validateCassandra = validateCassandraConfig(cassandraConfig);
	if (!validateCassandra.isValid) {
		echo.error("Cassandra configuration validation failed:");
		for (const error of validateCassandra.errors) {
			echo.error(`- ${error}`);
		}
		hasError = true;
	}

	const validateJWT = validateJWTConfig(jwt);
	if (!validateJWT.isValid) {
		echo.error("JWT configuration validation failed:");
		for (const error of validateJWT.errors) {
			echo.error(`- ${error}`);
		}
		hasError = true;
	}

	const validateMailer = validateMailerConfig(mailerConfig);
	if (!validateMailer.isValid) {
		echo.error("Mailer configuration validation failed:");
		for (const error of validateMailer.errors) {
			echo.error(`- ${error}`);
		}
		hasError = true;
	}

	const validateUrl = (url: string, name: string) => {
		const validation = isValidUrl(url, {
			requireProtocol: true,
			failOnTrailingSlash: true,
			allowedProtocols: ["http", "https"],
			allowLocalhost: environment.development,
			allowIP: environment.development,
		});

		if (!validation.valid) {
			echo.error(`${name} validation failed:`);
			echo.error(`- ${validation.error}`);
			hasError = true;
		}
	};

	validateUrl(environment.frontendFqdn, "FRONTEND_FQDN");
	validateUrl(environment.backendFqdn, "BACKEND_FQDN");

	if (hasError) {
		process.exit(1);
	}
}

export { environment, verifyRequiredVariables };
