import { readdirSync } from "node:fs";
import { Echo } from "@atums/echo";
import nodemailer from "nodemailer";
import { templatesPath } from "#environment/constants";
import { mailerConfig } from "#environment/mailer";

import type {
	EmailResult,
	SendMailOptions,
	TemplateVariables,
} from "#types/lib";

class MailerService {
	private transporter: nodemailer.Transporter;
	private templates: string[] = [];
	private static logger: Echo = new Echo({ subDirectory: "mailer" });
	private static loggerNoFile: Echo = new Echo({ disableFile: true });

	constructor() {
		this.transporter = nodemailer.createTransport({
			host: mailerConfig.address,
			port: mailerConfig.port,
			secure: mailerConfig.secure,
			auth: {
				user: mailerConfig.username,
				pass: mailerConfig.password,
			},
			pool: mailerConfig.pool,
			maxConnections: mailerConfig.maxConnections,
			maxMessages: mailerConfig.maxMessages,
			connectionTimeout: mailerConfig.connectionTimeout,
			socketTimeout: mailerConfig.socketTimeout,
			requireTLS: !mailerConfig.secure,
			tls: {
				rejectUnauthorized: true,
			},
		} as nodemailer.TransportOptions);

		this.populateTemplates();
	}

	async verifyConnection(): Promise<boolean> {
		try {
			await this.transporter.verify();
			MailerService.loggerNoFile.info("SMTP connection verified successfully");
			return true;
		} catch (error) {
			MailerService.logger.error({
				message: "SMTP connection verification failed:",
				error,
			});
			return false;
		}
	}

	async close(): Promise<void> {
		this.transporter.close();
	}

	public populateTemplates(): void {
		try {
			const files = readdirSync(templatesPath, { recursive: true });
			this.templates = files
				.filter((file) => typeof file === "string" && file.endsWith(".html"))
				.map((file) => (file as string).replace(/\.html$/, ""));
		} catch (error) {
			MailerService.logger.error({
				message: "Failed to list templates:",
				error,
			});
			this.templates = [];
		}
	}

	private async renderTemplate(
		templateName: string,
		variables: TemplateVariables,
	): Promise<string> {
		const templates = this.templates;
		if (!templates.includes(templateName)) {
			throw new Error(`Template "${templateName}" not found`);
		}

		const templatePath = `${templatesPath}/${templateName}.html`;
		let templateContent: string;

		try {
			const file = Bun.file(templatePath);
			templateContent = await file.text();
		} catch (error) {
			MailerService.logger.error({
				message: `Failed to load template "${templateName}":`,
				error,
			});
			throw error;
		}

		let processedContent = templateContent;
		for (const [key, value] of Object.entries(variables)) {
			const regex = new RegExp(`{{${key}}}`, "g");
			processedContent = processedContent.replace(regex, String(value));
		}

		return processedContent;
	}

	async sendEmail(options: SendMailOptions): Promise<EmailResult> {
		try {
			let htmlContent: string;
			let textContent: string | undefined;

			if (options.template && options.templateVariables) {
				htmlContent = await this.renderTemplate(
					options.template,
					options.templateVariables,
				);

				if (!options.text) {
					textContent = this.htmlToText(htmlContent);
				} else {
					textContent = options.text;
				}
			} else if (options.html) {
				htmlContent = options.html;
				textContent = options.text;
			} else {
				throw new Error("Either template or html content must be provided");
			}

			const mailOptions: nodemailer.SendMailOptions = {
				from: options.from || mailerConfig.from,
				to: Array.isArray(options.to) ? options.to.join(", ") : options.to,
				cc: options.cc
					? Array.isArray(options.cc)
						? options.cc.join(", ")
						: options.cc
					: undefined,
				bcc: options.bcc
					? Array.isArray(options.bcc)
						? options.bcc.join(", ")
						: options.bcc
					: undefined,
				subject: options.subject,
				text: textContent,
				html: htmlContent,
				attachments: options.attachments,
				replyTo: options.replyTo,
				priority: options.priority,
				headers: options.headers,
			};

			const info = await this.transporter.sendMail(mailOptions);

			MailerService.logger.debug({
				message: `Email sent successfully to ${options.to}`,
				messageId: info.messageId,
				subject: options.subject,
				template: options.template,
			});

			return {
				success: true,
				messageId: info.messageId,
				response: info.response,
			};
		} catch (error) {
			MailerService.logger.error({
				message: "Failed to send email:",
				error,
				to: options.to,
				subject: options.subject,
				template: options.template,
			});

			return {
				success: false,
				error:
					error instanceof Error ? error.message : "Unknown error occurred",
			};
		}
	}

	async sendTemplateEmail(
		to: string | string[],
		subject: string,
		templateName: string,
		variables: TemplateVariables,
		options?: Partial<SendMailOptions>,
	): Promise<EmailResult> {
		return this.sendEmail({
			to,
			subject,
			template: templateName,
			templateVariables: variables,
			...options,
		});
	}

	private htmlToText(html: string): string {
		return html
			.replace(/<style[^>]*>.*?<\/style>/gis, "") // remove style tags
			.replace(/<script[^>]*>.*?<\/script>/gis, "") // remove script tags
			.replace(/<[^>]*>/g, "") // remove HTML tags
			.replace(/&nbsp;/g, " ") // replace non-breaking spaces
			.replace(/&amp;/g, "&") // replace ampersands
			.replace(/&lt;/g, "<") // replace less than
			.replace(/&gt;/g, ">") // replace greater than
			.replace(/&quot;/g, '"') // replace quotes
			.replace(/&#39;/g, "'") // replace apostrophes
			.replace(/\s+/g, " ") // replace multiple whitespace with single space
			.trim();
	}
}

const mailerService = new MailerService();
export { MailerService, mailerService };
