function formatSecondsToHighestUnit(
	seconds: number,
	options: {
		capitalize?: boolean;
		pluralize?: boolean;
		short?: boolean;
	} = {},
): string {
	const { capitalize = false, pluralize = true, short = false } = options;

	const timeUnits = [
		{
			value: 365 * 24 * 60 * 60,
			singular: "year",
			plural: "years",
			short: "y",
		},
		{
			value: 30 * 24 * 60 * 60,
			singular: "month",
			plural: "months",
			short: "mo",
		},
		{ value: 7 * 24 * 60 * 60, singular: "week", plural: "weeks", short: "w" },
		{ value: 24 * 60 * 60, singular: "day", plural: "days", short: "d" },
		{ value: 60 * 60, singular: "hour", plural: "hours", short: "h" },
		{ value: 60, singular: "minute", plural: "minutes", short: "m" },
		{ value: 1, singular: "second", plural: "seconds", short: "s" },
	];

	if (seconds === 0) {
		const unit = short ? "s" : pluralize ? "seconds" : "second";
		return `0 ${capitalize ? capitalizeFirst(unit) : unit}`;
	}

	if (seconds < 0) {
		return formatSecondsToHighestUnit(Math.abs(seconds), options);
	}

	for (const unit of timeUnits) {
		const count = Math.floor(seconds / unit.value);
		if (count >= 1) {
			let unitName: string;

			if (short) {
				unitName = unit.short;
			} else if (pluralize && count !== 1) {
				unitName = unit.plural;
			} else {
				unitName = unit.singular;
			}

			if (capitalize && !short) {
				unitName = capitalizeFirst(unitName);
			}

			return `${count} ${unitName}`;
		}
	}

	const unit = short ? "s" : pluralize ? "seconds" : "second";
	return `${seconds} ${capitalize ? capitalizeFirst(unit) : unit}`;
}

function capitalizeFirst(str: string): string {
	return str.charAt(0).toUpperCase() + str.slice(1);
}

function formatSecondsToMultipleUnits(
	seconds: number,
	maxUnits = 2,
	options: {
		capitalize?: boolean;
		short?: boolean;
		separator?: string;
	} = {},
): string {
	const { capitalize = true, short = false, separator = " " } = options;

	const timeUnits = [
		{
			value: 365 * 24 * 60 * 60,
			singular: "year",
			plural: "years",
			short: "y",
		},
		{
			value: 30 * 24 * 60 * 60,
			singular: "month",
			plural: "months",
			short: "mo",
		},
		{ value: 7 * 24 * 60 * 60, singular: "week", plural: "weeks", short: "w" },
		{ value: 24 * 60 * 60, singular: "day", plural: "days", short: "d" },
		{ value: 60 * 60, singular: "hour", plural: "hours", short: "h" },
		{ value: 60, singular: "minute", plural: "minutes", short: "m" },
		{ value: 1, singular: "second", plural: "seconds", short: "s" },
	];

	if (seconds === 0) {
		const unit = short ? "s" : "seconds";
		return `0 ${capitalize ? capitalizeFirst(unit) : unit}`;
	}

	const parts: string[] = [];
	let remaining = Math.abs(seconds);

	for (const unit of timeUnits) {
		if (parts.length >= maxUnits) break;

		const count = Math.floor(remaining / unit.value);
		if (count > 0) {
			let unitName: string;

			if (short) {
				unitName = unit.short;
			} else {
				unitName = count === 1 ? unit.singular : unit.plural;
			}

			if (capitalize && !short) {
				unitName = capitalizeFirst(unitName);
			}

			parts.push(`${count} ${unitName}`);
			remaining -= count * unit.value;
		}
	}

	return parts.join(separator) || "0 seconds";
}

export { formatSecondsToHighestUnit, formatSecondsToMultipleUnits };
