function isValidPort(port: number): boolean {
	return Number.isInteger(port) && port > 0 && port <= 65535;
}

function isIPAddress(hostname: string): boolean {
	// IPv4
	const ipv4Regex =
		/^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/;

	// IPv6 (simplified)
	const ipv6Regex = /^(?:[0-9a-fA-F]{1,4}:){7}[0-9a-fA-F]{1,4}$|^::1$|^::$/;

	return ipv4Regex.test(hostname) || ipv6Regex.test(hostname);
}

function isValidHostname(hostname: string): boolean {
	if (!hostname || hostname.length === 0 || hostname.length > 253) {
		return false;
	}

	// check for valid hostname format
	const hostnameRegex =
		/^[a-zA-Z0-9]([a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?(\.[a-zA-Z0-9]([a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)*$/;

	if (!hostnameRegex.test(hostname)) {
		return false;
	}

	if (hostname.startsWith("-") || hostname.endsWith("-")) {
		return false;
	}

	const labels = hostname.split(".");
	for (const label of labels) {
		if (label.startsWith("-") || label.endsWith("-") || label.length > 63) {
			return false;
		}
	}

	return true;
}

function hasSuspiciousPatterns(url: string): boolean {
	// check for multiple dots in a row
	if (/\.{2,}/.test(url)) return true;

	// check for suspicious URL encoding
	if (/%[0-9a-fA-F]{2}%[0-9a-fA-F]{2}/.test(url)) return true;

	// check for double slashes in path (excluding protocol)
	if (/(?<!:)\/\//.test(url)) return true;

	// check for control characters using character codes
	for (let i = 0; i < url.length; i++) {
		const charCode = url.charCodeAt(i);

		// C0 control characters (0x00-0x1F) and DEL + C1 control characters (0x7F-0x9F)
		if (
			(charCode >= 0x00 && charCode <= 0x1f) ||
			(charCode >= 0x7f && charCode <= 0x9f)
		) {
			return true;
		}

		// unicode zero-width and formatting characters
		if (
			charCode === 0x200b ||
			charCode === 0x200c ||
			charCode === 0x200d ||
			charCode === 0xfeff
		) {
			return true;
		}
	}

	return false;
}

function isValidLength(value: string, min: number, max: number): boolean {
	return value.length >= min && value.length <= max;
}

function isNonEmptyString(value: string): boolean {
	return typeof value === "string" && value.trim().length > 0;
}

function normalizeString(value: string): string {
	return value.trim().normalize("NFC");
}

export {
	isValidPort,
	isIPAddress,
	isValidHostname,
	hasSuspiciousPatterns,
	isValidLength,
	isNonEmptyString,
	normalizeString,
};
