import type { JWTConfig } from "#types/config";
import type { simpleConfigValidation } from "#types/lib";

function isValidSecret(secret: string): boolean {
	if (!secret || secret.trim().length === 0) return false;
	return secret.length >= 32;
}

function isValidExpiration(expiration: string): boolean {
	if (!expiration || expiration.trim().length === 0) return false;
	const timeFormatRegex = /^(\d+)([smhdwy])$/;
	return timeFormatRegex.test(expiration.toLowerCase());
}

function isValidIssuer(issuer: string): boolean {
	if (!issuer || issuer.trim().length === 0) return false;
	const issuerRegex = /^[a-zA-Z0-9]([a-zA-Z0-9-_.])*[a-zA-Z0-9]$/;
	return issuer.length <= 255 && issuerRegex.test(issuer);
}

function isValidAlgorithm(algorithm: string): boolean {
	const supportedAlgorithms = [
		"HS256",
		"HS384",
		"HS512",
		"RS256",
		"RS384",
		"RS512",
		"ES256",
		"ES384",
		"ES512",
		"PS256",
		"PS384",
		"PS512",
	];
	return supportedAlgorithms.includes(algorithm);
}

function validateJWTConfig(config: JWTConfig): simpleConfigValidation {
	const errors: string[] = [];

	if (!isValidSecret(config.secret)) {
		errors.push("Invalid JWT secret: Must be at least 32 characters long");
	}

	const expirationStr =
		typeof config.expiration === "number"
			? `${config.expiration}s`
			: config.expiration;

	if (!isValidExpiration(expirationStr)) {
		errors.push(
			"Invalid JWT expiration: Must be in format like '1h', '30m', '7d', '1y'",
		);
	}

	if (!isValidIssuer(config.issuer)) {
		errors.push(
			"Invalid JWT issuer: Must be a valid identifier (domain, URL, or app name)",
		);
	}

	if (!isValidAlgorithm(config.algorithm)) {
		errors.push(
			`Invalid JWT algorithm: ${config.algorithm}. Must be one of: HS256, HS384, HS512, RS256, RS384, RS512, ES256, ES384, ES512, PS256, PS384, PS512`,
		);
	}

	return {
		isValid: errors.length === 0,
		...(errors.length > 0 && { error: errors.join("; ") }),
		errors,
	};
}

export {
	isValidSecret,
	isValidExpiration,
	isValidIssuer,
	isValidAlgorithm,
	validateJWTConfig,
};
