import { hasSuspiciousPatterns, isIPAddress, isValidHostname } from "./general";

import type { UrlValidationOptions, UrlValidationResult } from "#types/lib";

function isValidUrl(
	rawUrl: string,
	options: UrlValidationOptions = {},
): UrlValidationResult {
	const {
		failOnTrailingSlash = false,
		removeTrailingSlash = false,
		allowedProtocols = ["http", "https"],
		requireProtocol = true,
		allowLocalhost = true,
		allowIP = true,
		maxLength = 2048,
	} = options;

	if (typeof rawUrl !== "string") {
		return { valid: false, error: "URL must be a string" };
	}

	const url = rawUrl.trim();

	if (!url) {
		return { valid: false, error: "URL is required" };
	}

	if (url.length > maxLength) {
		return {
			valid: false,
			error: `URL exceeds maximum length of ${maxLength} characters`,
		};
	}

	let urlToValidate = url;

	// add protocol if missing and required
	if (requireProtocol && !url.match(/^[a-zA-Z][a-zA-Z0-9+.-]*:/)) {
		urlToValidate = `https://${url}`;
	}

	let parsedUrl: URL;
	try {
		parsedUrl = new URL(urlToValidate);
	} catch {
		return { valid: false, error: "Invalid URL format" };
	}

	const protocol = parsedUrl.protocol.slice(0, -1);
	if (!allowedProtocols.includes(protocol)) {
		return {
			valid: false,
			error: `Invalid protocol: ${protocol}. Allowed protocols: ${allowedProtocols.join(", ")}`,
		};
	}

	const hostname = parsedUrl.hostname.toLowerCase();

	if (
		!allowLocalhost &&
		(hostname === "localhost" || hostname === "127.0.0.1")
	) {
		return { valid: false, error: "Localhost URLs are not allowed" };
	}

	if (!allowIP && isIPAddress(hostname)) {
		return { valid: false, error: "IP addresses are not allowed" };
	}

	if (!isValidHostname(hostname) && !isIPAddress(hostname)) {
		return { valid: false, error: "Invalid hostname format" };
	}

	if (hasSuspiciousPatterns(url)) {
		return { valid: false, error: "URL contains suspicious patterns" };
	}

	const originalHasTrailingSlash =
		urlToValidate.endsWith("/") && !urlToValidate.endsWith("://");

	if (originalHasTrailingSlash && failOnTrailingSlash) {
		return {
			valid: false,
			error: "URL must not end with a trailing slash",
		};
	}

	let normalizedUrl = parsedUrl.toString();

	const normalizedHasTrailingSlash =
		normalizedUrl.endsWith("/") && normalizedUrl !== `${parsedUrl.origin}/`;

	if (normalizedHasTrailingSlash && removeTrailingSlash) {
		normalizedUrl = normalizedUrl.slice(0, -1);
	}

	return {
		valid: true,
		url: urlToValidate,
		normalizedUrl,
	};
}

function normalizeUrl(
	url: string,
	options: UrlValidationOptions = {},
): string | null {
	const result = isValidUrl(url, options);
	return result.valid ? result.normalizedUrl || result.url || null : null;
}

export { isValidUrl, normalizeUrl };
