import {
	displayNameRestrictions,
	forbiddenDisplayNamePatterns,
	nameRestrictions,
	reservedNames,
} from "#environment/constants";

import type { validationResult } from "#types/lib";

function isValidUsername(rawUsername: string): validationResult {
	if (typeof rawUsername !== "string") {
		return { valid: false, error: "Username must be a string" };
	}

	const username = rawUsername.trim().normalize("NFC");

	if (!username) return { valid: false, error: "Username is required" };

	if (username.length < nameRestrictions.length.min)
		return { valid: false, error: "Username is too short" };

	if (username.length > nameRestrictions.length.max)
		return { valid: false, error: "Username is too long" };

	if (!nameRestrictions.regex.test(username))
		return { valid: false, error: "Username contains invalid characters" };

	if (/^[._-]|[._-]$/.test(username)) {
		return {
			valid: false,
			error: "Username can't start or end with special characters",
		};
	}

	if (reservedNames.includes(username.toLowerCase())) {
		return {
			valid: false,
			error: "Username is reserved and cannot be used",
		};
	}

	return { valid: true, username };
}

function isValidDisplayName(rawDisplayName: string): validationResult {
	if (typeof rawDisplayName !== "string") {
		return { valid: false, error: "Display name must be a string" };
	}

	const displayName = rawDisplayName.normalize("NFC");

	if (!displayName) {
		return { valid: false, error: "Display name is required" };
	}

	if (displayName.length < displayNameRestrictions.length.min) {
		return { valid: false, error: "Display name is too short" };
	}

	if (displayName.length > displayNameRestrictions.length.max) {
		return { valid: false, error: "Display name is too long" };
	}

	for (const pattern of forbiddenDisplayNamePatterns) {
		if (pattern.test(displayName)) {
			return {
				valid: false,
				error: "Display name contains invalid characters or patterns",
			};
		}
	}

	if (!displayNameRestrictions.regex.test(displayName)) {
		return {
			valid: false,
			error: "Display name contains invalid characters",
		};
	}

	if (displayName.trim().length === 0) {
		return {
			valid: false,
			error: "Display name cannot be only whitespace",
		};
	}

	return { valid: true, name: displayName };
}

export { isValidUsername, isValidDisplayName };
