import { echo } from "@atums/echo";
import { errorMessages, httpStatus } from "#environment/constants";
import { cassandra } from "#lib/database";

import type {
	ExtendedRequest,
	RouteDef,
	UserInfoResponse,
	UserResponse,
	UserRow,
} from "#types/server";

const routeDef: RouteDef = {
	method: "GET",
	accepts: "*/*",
	returns: "application/json",
};

async function handler(request: ExtendedRequest): Promise<Response> {
	try {
		const { id: identifier } = request.params;
		const { session } = request;

		let userQuery: string;
		let queryParams: string[];
		let targetUser: UserRow | null = null;

		if (!identifier) {
			if (!session) {
				const response: UserInfoResponse = {
					code: httpStatus.UNAUTHORIZED,
					success: false,
					error: errorMessages.NOT_AUTHENTICATED,
				};
				return Response.json(response, { status: httpStatus.UNAUTHORIZED });
			}

			userQuery = `
				SELECT id, username, display_name, email, is_verified, created_at, updated_at
				FROM users WHERE id = ? LIMIT 1
			`;
			queryParams = [session.id];
		} else {
			const isLikelyId = identifier.startsWith("user_");

			if (isLikelyId) {
				userQuery = `
					SELECT id, username, display_name, email, is_verified, created_at, updated_at
					FROM users WHERE id = ? LIMIT 1
				`;
				queryParams = [identifier];
			} else {
				userQuery = `
					SELECT id, username, display_name, email, is_verified, created_at, updated_at
					FROM users WHERE username = ? LIMIT 1
				`;
				queryParams = [identifier];
			}
		}

		const userResult = (await cassandra.execute(userQuery, queryParams)) as {
			rows: UserRow[];
		};

		if (!userResult?.rows || !Array.isArray(userResult.rows)) {
			const response: UserInfoResponse = {
				code: httpStatus.INTERNAL_SERVER_ERROR,
				success: false,
				error: errorMessages.DATABASE_QUERY_FAILED,
			};
			return Response.json(response, {
				status: httpStatus.INTERNAL_SERVER_ERROR,
			});
		}

		if (userResult.rows.length === 0) {
			if (identifier?.startsWith("user_")) {
				const usernameQuery = `
					SELECT id, username, display_name, email, is_verified, created_at, updated_at
					FROM users WHERE username = ? LIMIT 1
				`;

				const usernameResult = (await cassandra.execute(usernameQuery, [
					identifier,
				])) as {
					rows: UserRow[];
				};

				if (usernameResult.rows.length > 0) {
					targetUser = usernameResult.rows[0] || null;
				}
			}

			if (!targetUser) {
				const response: UserInfoResponse = {
					code: httpStatus.NOT_FOUND,
					success: false,
					error: errorMessages.USER_NOT_FOUND,
				};
				return Response.json(response, { status: httpStatus.NOT_FOUND });
			}
		} else {
			targetUser = userResult.rows[0] || null;
		}

		if (!targetUser) {
			const response: UserInfoResponse = {
				code: httpStatus.NOT_FOUND,
				success: false,
				error: errorMessages.USER_NOT_FOUND,
			};
			return Response.json(response, { status: httpStatus.NOT_FOUND });
		}

		const isOwnProfile = session?.id === targetUser.id;

		let responseUser: UserResponse;

		if (isOwnProfile) {
			responseUser = {
				id: targetUser.id,
				username: targetUser.username,
				displayName: targetUser.display_name,
				email: targetUser.email,
				isVerified: targetUser.is_verified,
				createdAt: targetUser.created_at.toISOString(),
			};
		} else {
			responseUser = {
				id: targetUser.id,
				username: targetUser.username,
				displayName: targetUser.display_name,
				email: "",
				isVerified: targetUser.is_verified,
				createdAt: targetUser.created_at.toISOString(),
			};
		}

		const response: UserInfoResponse = {
			code: httpStatus.OK,
			success: true,
			message: isOwnProfile
				? "User information retrieved successfully"
				: "Public user information retrieved successfully",
			user: responseUser,
		};

		return Response.json(response, { status: httpStatus.OK });
	} catch (error) {
		echo.error({
			message: "Error retrieving user information",
			error,
		});

		const response: UserInfoResponse = {
			code: httpStatus.INTERNAL_SERVER_ERROR,
			success: false,
			error: errorMessages.INTERNAL_SERVER_ERROR,
		};
		return Response.json(response, {
			status: httpStatus.INTERNAL_SERVER_ERROR,
		});
	}
}

export { handler, routeDef };
