import { echo } from "@atums/echo";
import { redis } from "bun";
import { environment } from "#environment/config";
import {
	cacheTTL,
	errorMessages,
	generateCacheKey,
	httpStatus,
	successMessages,
} from "#environment/constants";
import { extraValues } from "#environment/extra";
import { cassandra } from "#lib/database";
import { mailerService } from "#lib/mailer";
import { formatSecondsToHighestUnit } from "#lib/utils";
import { isValidEmail } from "#lib/validation";

import type {
	BaseResponse,
	ExtendedRequest,
	ForgotPasswordRequest,
	RouteDef,
} from "#types/server";

const routeDef: RouteDef = {
	method: "POST",
	accepts: "application/json",
	returns: "application/json",
	needsBody: "json",
};

async function handler(request: ExtendedRequest): Promise<Response> {
	try {
		const { email } = request.requestBody as ForgotPasswordRequest;

		if (!email) {
			const response: BaseResponse = {
				code: httpStatus.BAD_REQUEST,
				success: false,
				error: "Email is required",
			};
			return Response.json(response, { status: httpStatus.BAD_REQUEST });
		}

		const emailValidation = isValidEmail(email);
		if (!emailValidation.valid) {
			const response: BaseResponse = {
				code: httpStatus.BAD_REQUEST,
				success: false,
				error: emailValidation.error || "Invalid email",
			};
			return Response.json(response, { status: httpStatus.BAD_REQUEST });
		}

		const userQuery = `
			SELECT id, username, display_name, email, is_verified
			FROM users WHERE email = ? LIMIT 1
		`;
		const userResult = (await cassandra.execute(userQuery, [
			email.trim().toLowerCase(),
		])) as {
			rows: Array<{
				id: string;
				username: string;
				display_name: string | null;
				email: string;
				is_verified: boolean;
			}>;
		};

		if (!userResult?.rows || userResult.rows.length === 0) {
			const response: BaseResponse = {
				code: httpStatus.OK,
				success: true,
				message: successMessages.PASSWORD_RESET_SENT,
			};
			return Response.json(response, { status: httpStatus.OK });
		}

		const user = userResult.rows[0];
		if (!user) {
			const response: BaseResponse = {
				code: httpStatus.OK,
				success: true,
				message: successMessages.PASSWORD_RESET_SENT,
			};
			return Response.json(response, { status: httpStatus.OK });
		}

		const resetToken = Bun.randomUUIDv7();
		const resetKey = generateCacheKey.passwordReset(resetToken);

		try {
			await redis.set(
				resetKey,
				JSON.stringify({
					userId: user.id,
					email: user.email,
				}),
				"EX",
				cacheTTL.passwordReset,
			);

			const emailVariables = {
				subject: `Password Reset - ${extraValues.companyName}`,
				companyName: extraValues.companyName,
				id: user.id,
				displayName: user.display_name || user.username,
				willExpire: `This link will expire in ${formatSecondsToHighestUnit(cacheTTL.passwordReset)}`,
				resetUrl: `${environment.frontendFqdn}/user/reset-password?token=${resetToken}`,
				supportEmail: extraValues.supportEmail,
			};

			await mailerService.sendTemplateEmail(
				user.email,
				`Password Reset - ${extraValues.companyName}`,
				"forgot-password",
				emailVariables,
			);

			const response: BaseResponse = {
				code: httpStatus.OK,
				success: true,
				message: successMessages.PASSWORD_RESET_SENT,
			};
			return Response.json(response, { status: httpStatus.OK });
		} catch (error) {
			await redis.del(resetKey).catch(() => {});

			echo.error({
				message: "Failed to send password reset email",
				error,
				userId: user.id,
				email: user.email,
			});

			const response: BaseResponse = {
				code: httpStatus.INTERNAL_SERVER_ERROR,
				success: false,
				error: errorMessages.EMAIL_SEND_FAILED,
			};
			return Response.json(response, {
				status: httpStatus.INTERNAL_SERVER_ERROR,
			});
		}
	} catch (error) {
		echo.error({
			message: "Password reset request failed",
			error,
		});

		const response: BaseResponse = {
			code: httpStatus.INTERNAL_SERVER_ERROR,
			success: false,
			error: errorMessages.INTERNAL_SERVER_ERROR,
		};
		return Response.json(response, {
			status: httpStatus.INTERNAL_SERVER_ERROR,
		});
	}
}

export { handler, routeDef };
