import { echo } from "@atums/echo";
import { redis } from "bun";
import {
	errorMessages,
	generateCacheKey,
	httpStatus,
	passwordHashing,
} from "#environment/constants";
import { sessionManager } from "#lib/auth";
import { cassandra } from "#lib/database";
import { isValidPassword } from "#lib/validation";

import type {
	ExtendedRequest,
	ResetData,
	ResetPasswordRequest,
	ResetPasswordResponse,
	RouteDef,
	UserRow,
} from "#types/server";

const routeDef: RouteDef = {
	method: "POST",
	accepts: "application/json",
	returns: "application/json",
	needsBody: "json",
};

async function handler(request: ExtendedRequest): Promise<Response> {
	try {
		const {
			token,
			newPassword,
			logoutAllSessions = true,
		} = request.requestBody as ResetPasswordRequest;

		if (!token || !newPassword) {
			const response: ResetPasswordResponse = {
				code: httpStatus.BAD_REQUEST,
				success: false,
				error: "Token and new password are required",
			};
			return Response.json(response, { status: httpStatus.BAD_REQUEST });
		}

		const passwordValidation = isValidPassword(newPassword);
		if (!passwordValidation.valid) {
			const response: ResetPasswordResponse = {
				code: httpStatus.BAD_REQUEST,
				success: false,
				error: passwordValidation.error || "Invalid password",
			};
			return Response.json(response, { status: httpStatus.BAD_REQUEST });
		}

		const resetKey = generateCacheKey.passwordReset(token);
		const resetDataRaw = await redis.get(resetKey);

		if (!resetDataRaw) {
			const response: ResetPasswordResponse = {
				code: httpStatus.BAD_REQUEST,
				success: false,
				error: errorMessages.INVALID_TOKEN,
			};
			return Response.json(response, { status: httpStatus.BAD_REQUEST });
		}

		let resetData: ResetData;
		try {
			resetData = JSON.parse(resetDataRaw);
		} catch {
			await redis.del(resetKey);
			const response: ResetPasswordResponse = {
				code: httpStatus.BAD_REQUEST,
				success: false,
				error: errorMessages.INVALID_TOKEN_FORMAT,
			};
			return Response.json(response, { status: httpStatus.BAD_REQUEST });
		}

		if (!resetData.userId || !resetData.email) {
			await redis.del(resetKey);
			const response: ResetPasswordResponse = {
				code: httpStatus.BAD_REQUEST,
				success: false,
				error: "Invalid reset data",
			};
			return Response.json(response, { status: httpStatus.BAD_REQUEST });
		}

		const userQuery = `
			SELECT id, username, display_name, email, password, is_verified, created_at, updated_at
			FROM users WHERE id = ? LIMIT 1
		`;
		const userResult = (await cassandra.execute(userQuery, [
			resetData.userId,
		])) as { rows: UserRow[] };

		if (!userResult?.rows || userResult.rows.length === 0) {
			await redis.del(resetKey);
			const response: ResetPasswordResponse = {
				code: httpStatus.NOT_FOUND,
				success: false,
				error: errorMessages.USER_NOT_FOUND,
			};
			return Response.json(response, { status: httpStatus.NOT_FOUND });
		}

		const user = userResult.rows[0];
		if (!user) {
			await redis.del(resetKey);
			const response: ResetPasswordResponse = {
				code: httpStatus.NOT_FOUND,
				success: false,
				error: errorMessages.USER_NOT_FOUND,
			};
			return Response.json(response, { status: httpStatus.NOT_FOUND });
		}

		if (user.email !== resetData.email) {
			await redis.del(resetKey);
			const response: ResetPasswordResponse = {
				code: httpStatus.BAD_REQUEST,
				success: false,
				error: "Reset token does not match current email address",
			};
			return Response.json(response, { status: httpStatus.BAD_REQUEST });
		}

		const isSamePassword = await Bun.password.verify(
			newPassword,
			user.password,
		);
		if (isSamePassword) {
			const response: ResetPasswordResponse = {
				code: httpStatus.BAD_REQUEST,
				success: false,
				error: errorMessages.PASSWORD_SAME_AS_CURRENT,
			};
			return Response.json(response, { status: httpStatus.BAD_REQUEST });
		}

		const hashedNewPassword = await Bun.password.hash(
			newPassword,
			passwordHashing,
		);

		const updateQuery = `
			UPDATE users
			SET password = ?, updated_at = ?
			WHERE id = ?
		`;
		await cassandra.execute(updateQuery, [
			hashedNewPassword,
			new Date(),
			user.id,
		]);

		await redis.del(resetKey);

		let invalidatedCount = 0;
		if (logoutAllSessions) {
			invalidatedCount = await sessionManager.invalidateAllSessionsForUser(
				user.id,
			);
		}

		const baseMessage = "Password reset successfully";
		const sessionMessage = logoutAllSessions
			? `. Logged out from ${invalidatedCount} session(s).`
			: ".";

		const response: ResetPasswordResponse = {
			code: httpStatus.OK,
			success: true,
			message: baseMessage + sessionMessage,
			loggedOutSessions: invalidatedCount,
		};

		return Response.json(response, { status: httpStatus.OK });
	} catch (error) {
		echo.error({
			message: "Password reset failed",
			error,
		});

		const response: ResetPasswordResponse = {
			code: httpStatus.INTERNAL_SERVER_ERROR,
			success: false,
			error: errorMessages.INTERNAL_SERVER_ERROR,
		};
		return Response.json(response, {
			status: httpStatus.INTERNAL_SERVER_ERROR,
		});
	}
}

export { handler, routeDef };
