import { echo } from "@atums/echo";
import { redis } from "bun";
import { environment } from "#environment/config";
import {
	cacheTTL,
	errorMessages,
	generateCacheKey,
	httpStatus,
} from "#environment/constants";
import { extraValues } from "#environment/extra";
import { sessionManager } from "#lib/auth";
import { cassandra } from "#lib/database";
import { mailerService } from "#lib/mailer";
import { formatSecondsToHighestUnit } from "#lib/utils";
import { isValidEmail } from "#lib/validation";

import type { UserSession } from "#types/config";
import type {
	EmailChangeData,
	EmailChangeRequest,
	EmailChangeResponse,
	ExtendedRequest,
	RouteDef,
	UserResponse,
	UserRow,
} from "#types/server";

const routeDef: RouteDef = {
	method: ["POST", "GET"],
	accepts: ["application/json", "*/*"],
	returns: "application/json",
	needsBody: "json",
};

async function handler(request: ExtendedRequest): Promise<Response> {
	try {
		const { session } = request;

		if (!session) {
			const response: EmailChangeResponse = {
				code: httpStatus.UNAUTHORIZED,
				success: false,
				error: errorMessages.NOT_AUTHENTICATED,
			};
			return Response.json(response, { status: httpStatus.UNAUTHORIZED });
		}

		if (request.method === "GET") {
			return await handleEmailVerification(request, session);
		}

		return await handleEmailChangeRequest(request, session);
	} catch (error) {
		echo.error({
			message: "Email change operation failed",
			error,
		});

		const response: EmailChangeResponse = {
			code: httpStatus.INTERNAL_SERVER_ERROR,
			success: false,
			error: errorMessages.INTERNAL_SERVER_ERROR,
		};
		return Response.json(response, {
			status: httpStatus.INTERNAL_SERVER_ERROR,
		});
	}
}

async function handleEmailChangeRequest(
	request: ExtendedRequest,
	session: UserSession,
): Promise<Response> {
	const { newEmail } = request.requestBody as EmailChangeRequest;

	if (!newEmail) {
		const response: EmailChangeResponse = {
			code: httpStatus.BAD_REQUEST,
			success: false,
			error: "New email is required",
		};
		return Response.json(response, { status: httpStatus.BAD_REQUEST });
	}

	const emailValidation = isValidEmail(newEmail);
	if (!emailValidation.valid) {
		const response: EmailChangeResponse = {
			code: httpStatus.BAD_REQUEST,
			success: false,
			error: emailValidation.error || "Invalid email",
		};
		return Response.json(response, { status: httpStatus.BAD_REQUEST });
	}

	const normalizedEmail = newEmail.trim().toLowerCase();

	const currentUserQuery = `
		SELECT id, username, display_name, email, is_verified, created_at, updated_at
		FROM users WHERE id = ? LIMIT 1
	`;
	const currentUserResult = (await cassandra.execute(currentUserQuery, [
		session.id,
	])) as { rows: UserRow[] };

	if (!currentUserResult?.rows || currentUserResult.rows.length === 0) {
		await sessionManager.invalidateSession(request);

		const response: EmailChangeResponse = {
			code: httpStatus.NOT_FOUND,
			success: false,
			error: errorMessages.USER_NOT_FOUND,
		};
		return Response.json(response, { status: httpStatus.NOT_FOUND });
	}

	const currentUser = currentUserResult.rows[0];
	if (!currentUser) {
		const response: EmailChangeResponse = {
			code: httpStatus.NOT_FOUND,
			success: false,
			error: errorMessages.USER_NOT_FOUND,
		};
		return Response.json(response, { status: httpStatus.NOT_FOUND });
	}

	if (normalizedEmail === currentUser.email) {
		const response: EmailChangeResponse = {
			code: httpStatus.BAD_REQUEST,
			success: false,
			error: "New email must be different from current email",
		};
		return Response.json(response, { status: httpStatus.BAD_REQUEST });
	}

	const cooldownKey = generateCacheKey.emailChangeCooldown(session.id);
	const lastRequest = await redis.get(cooldownKey);

	if (lastRequest) {
		const lastRequestTime = Number.parseInt(lastRequest, 10);
		const timeSince = Date.now() - lastRequestTime;
		const cooldownMs = cacheTTL.emailChangeCooldown * 1000;

		if (timeSince < cooldownMs) {
			const remainingMs = cooldownMs - timeSince;
			const remainingMinutes = Math.ceil(remainingMs / (60 * 1000));

			const response: EmailChangeResponse = {
				code: httpStatus.TOO_MANY_REQUESTS,
				success: false,
				error: `Please wait ${remainingMinutes} minute(s) before requesting another email change`,
				cooldownRemaining: remainingMinutes,
			};
			return Response.json(response, { status: httpStatus.TOO_MANY_REQUESTS });
		}
	}

	const existingEmailQuery = "SELECT id FROM users WHERE email = ? LIMIT 1";
	const existingEmailResult = (await cassandra.execute(existingEmailQuery, [
		normalizedEmail,
	])) as { rows: Array<{ id: string }> };

	if (existingEmailResult.rows.length > 0) {
		const response: EmailChangeResponse = {
			code: httpStatus.CONFLICT,
			success: false,
			error: errorMessages.EMAIL_ALREADY_EXISTS,
		};
		return Response.json(response, { status: httpStatus.CONFLICT });
	}

	const verificationToken = Bun.randomUUIDv7();
	const emailChangeKey = generateCacheKey.emailChange(verificationToken);
	const now = Date.now();

	try {
		const emailChangeData: EmailChangeData = {
			userId: currentUser.id,
			currentEmail: currentUser.email,
			newEmail: normalizedEmail,
			requestedAt: now,
		};

		await redis.set(
			emailChangeKey,
			JSON.stringify(emailChangeData),
			"EX",
			cacheTTL.emailChange,
		);

		await redis.set(
			cooldownKey,
			now.toString(),
			"EX",
			cacheTTL.emailChangeCooldown,
		);

		// send verification email to NEW email
		const emailVariables = {
			subject: `Email Change Verification - ${extraValues.companyName}`,
			companyName: extraValues.companyName,
			id: currentUser.id,
			displayName: currentUser.display_name || currentUser.username,
			currentEmail: currentUser.email,
			newEmail: normalizedEmail,
			willExpire: `This link will expire in ${formatSecondsToHighestUnit(cacheTTL.emailChange)}`,
			verificationUrl: `${environment.frontendFqdn}/user/email?token=${verificationToken}`,
			supportEmail: extraValues.supportEmail,
		};

		await mailerService.sendTemplateEmail(
			normalizedEmail,
			`Email Change Verification - ${extraValues.companyName}`,
			"email-change-verification",
			emailVariables,
		);

		// send notification email to OLD email about the change request
		const notificationVariables = {
			subject: `Email Change Request - ${extraValues.companyName}`,
			companyName: extraValues.companyName,
			displayName: currentUser.display_name || currentUser.username,
			id: currentUser.id,
			currentEmail: currentUser.email,
			newEmail: normalizedEmail,
			requestTime: new Date().toLocaleString(),
			willExpire: `This request will expire in ${formatSecondsToHighestUnit(cacheTTL.emailChange)}`,
			supportEmail: extraValues.supportEmail,
		};

		try {
			await mailerService.sendTemplateEmail(
				currentUser.email,
				`Email Change Request - ${extraValues.companyName}`,
				"email-change-request-notification",
				notificationVariables,
			);
		} catch (notificationError) {
			echo.warn({
				message: "Failed to send email change notification to old address",
				error: notificationError,
				userId: currentUser.id,
				currentEmail: currentUser.email,
			});
		}

		const response: EmailChangeResponse = {
			code: httpStatus.OK,
			success: true,
			message: `Email change verification sent to ${normalizedEmail}. Please check your new email to confirm the change. A notification has also been sent to your current email.`,
		};
		return Response.json(response, { status: httpStatus.OK });
	} catch (error) {
		await redis.del(emailChangeKey).catch(() => {});
		await redis.del(cooldownKey).catch(() => {});

		echo.error({
			message: "Failed to send email change verification",
			error,
			userId: currentUser.id,
			currentEmail: currentUser.email,
			newEmail: normalizedEmail,
		});

		const response: EmailChangeResponse = {
			code: httpStatus.INTERNAL_SERVER_ERROR,
			success: false,
			error: errorMessages.EMAIL_SEND_FAILED,
		};
		return Response.json(response, {
			status: httpStatus.INTERNAL_SERVER_ERROR,
		});
	}
}

async function handleEmailVerification(
	request: ExtendedRequest,
	session: UserSession,
): Promise<Response> {
	const { token } = request.query;

	if (!token || typeof token !== "string" || token.trim() === "") {
		const response: EmailChangeResponse = {
			code: httpStatus.BAD_REQUEST,
			success: false,
			error: "Email change verification token is required",
		};
		return Response.json(response, { status: httpStatus.BAD_REQUEST });
	}

	const emailChangeKey = generateCacheKey.emailChange(token);
	const emailChangeDataRaw = await redis.get(emailChangeKey);

	if (!emailChangeDataRaw) {
		const response: EmailChangeResponse = {
			code: httpStatus.BAD_REQUEST,
			success: false,
			error: errorMessages.INVALID_TOKEN,
		};
		return Response.json(response, { status: httpStatus.BAD_REQUEST });
	}

	let emailChangeData: EmailChangeData;
	try {
		emailChangeData = JSON.parse(emailChangeDataRaw);
	} catch {
		await redis.del(emailChangeKey);

		const response: EmailChangeResponse = {
			code: httpStatus.BAD_REQUEST,
			success: false,
			error: errorMessages.INVALID_TOKEN_FORMAT,
		};
		return Response.json(response, { status: httpStatus.BAD_REQUEST });
	}

	if (
		!emailChangeData.userId ||
		!emailChangeData.currentEmail ||
		!emailChangeData.newEmail
	) {
		await redis.del(emailChangeKey);

		const response: EmailChangeResponse = {
			code: httpStatus.BAD_REQUEST,
			success: false,
			error: "Invalid email change data",
		};
		return Response.json(response, { status: httpStatus.BAD_REQUEST });
	}

	const userQuery = `
		SELECT id, username, display_name, email, is_verified, created_at, updated_at
		FROM users WHERE id = ? LIMIT 1
	`;
	const userResult = (await cassandra.execute(userQuery, [
		emailChangeData.userId,
	])) as { rows: UserRow[] };

	if (!userResult?.rows || userResult.rows.length === 0) {
		await redis.del(emailChangeKey);

		const response: EmailChangeResponse = {
			code: httpStatus.NOT_FOUND,
			success: false,
			error: errorMessages.USER_NOT_FOUND,
		};
		return Response.json(response, { status: httpStatus.NOT_FOUND });
	}

	const user = userResult.rows[0];
	if (!user) {
		await redis.del(emailChangeKey);

		const response: EmailChangeResponse = {
			code: httpStatus.NOT_FOUND,
			success: false,
			error: errorMessages.USER_NOT_FOUND,
		};
		return Response.json(response, { status: httpStatus.NOT_FOUND });
	}

	if (user.email !== emailChangeData.currentEmail) {
		await redis.del(emailChangeKey);

		const response: EmailChangeResponse = {
			code: httpStatus.BAD_REQUEST,
			success: false,
			error:
				"Email change token is no longer valid - current email has changed",
		};
		return Response.json(response, { status: httpStatus.BAD_REQUEST });
	}

	const existingEmailQuery = "SELECT id FROM users WHERE email = ? LIMIT 1";
	const existingEmailResult = (await cassandra.execute(existingEmailQuery, [
		emailChangeData.newEmail,
	])) as { rows: Array<{ id: string }> };

	if (
		existingEmailResult.rows.length > 0 &&
		existingEmailResult.rows[0]?.id !== user.id
	) {
		await redis.del(emailChangeKey);

		const response: EmailChangeResponse = {
			code: httpStatus.CONFLICT,
			success: false,
			error: "New email address is no longer available",
		};
		return Response.json(response, { status: httpStatus.CONFLICT });
	}

	const updateQuery = `
		UPDATE users
		SET email = ?, is_verified = ?, updated_at = ?
		WHERE id = ?
	`;
	await cassandra.execute(updateQuery, [
		emailChangeData.newEmail,
		true,
		new Date(),
		user.id,
	]);

	await redis.del(emailChangeKey);

	try {
		const changeNotificationVariables = {
			subject: `Email Address Changed - ${extraValues.companyName}`,
			companyName: extraValues.companyName,
			displayName: user.display_name || user.username,
			id: user.id,
			oldEmail: emailChangeData.currentEmail,
			newEmail: emailChangeData.newEmail,
			changeTime: new Date().toLocaleString(),
			supportEmail: extraValues.supportEmail,
		};

		await mailerService.sendTemplateEmail(
			emailChangeData.currentEmail,
			`Email Address Changed - ${extraValues.companyName}`,
			"email-change-completed-notification",
			changeNotificationVariables,
		);
	} catch (notificationError) {
		echo.warn({
			message:
				"Failed to send email change completion notification to old address",
			error: notificationError,
			userId: user.id,
			oldEmail: emailChangeData.currentEmail,
			newEmail: emailChangeData.newEmail,
		});
	}

	const updatedUserResult = (await cassandra.execute(userQuery, [user.id])) as {
		rows: UserRow[];
	};

	const updatedUser = updatedUserResult.rows[0];
	if (!updatedUser) {
		const response: EmailChangeResponse = {
			code: httpStatus.INTERNAL_SERVER_ERROR,
			success: false,
			error: "Failed to fetch updated user data",
		};
		return Response.json(response, {
			status: httpStatus.INTERNAL_SERVER_ERROR,
		});
	}

	let sessionCookie: string | undefined;

	if (session && session.id === user.id) {
		try {
			const userAgent = request.headers.get("User-Agent") || "Unknown";
			const updatedSessionPayload = {
				id: updatedUser.id,
				username: updatedUser.username,
				email: updatedUser.email,
				isVerified: updatedUser.is_verified,
				displayName: updatedUser.display_name,
				createdAt: updatedUser.created_at.toISOString(),
				updatedAt: updatedUser.updated_at.toISOString(),
			};

			sessionCookie = await sessionManager.updateSession(
				request,
				updatedSessionPayload,
				userAgent,
			);
		} catch (sessionError) {
			echo.warn({
				message: "Failed to update session after email change",
				error: sessionError,
				userId: user.id,
			});
		}
	}

	const responseUser: UserResponse = {
		id: updatedUser.id,
		username: updatedUser.username,
		displayName: updatedUser.display_name,
		email: updatedUser.email,
		isVerified: updatedUser.is_verified,
		createdAt: updatedUser.created_at.toISOString(),
	};

	const response: EmailChangeResponse = {
		code: httpStatus.OK,
		success: true,
		message: `Email successfully changed to ${updatedUser.email} and verified.`,
		user: responseUser,
	};

	return Response.json(response, {
		status: httpStatus.OK,
		headers: sessionCookie ? { "Set-Cookie": sessionCookie } : {},
	});
}

export { handler, routeDef };
