import { echo } from "@atums/echo";
import {
	errorMessages,
	httpStatus,
	successMessages,
} from "#environment/constants";
import { sessionManager } from "#lib/auth";
import { cassandra } from "#lib/database";
import { isValidDisplayName, isValidUsername } from "#lib/validation";

import type {
	ExtendedRequest,
	RouteDef,
	UpdateInfoRequest,
	UpdateInfoResponse,
	UserResponse,
	UserRow,
} from "#types/server";

const routeDef: RouteDef = {
	method: ["PUT", "PATCH"],
	accepts: "application/json",
	returns: "application/json",
	needsBody: "json",
};

async function handler(request: ExtendedRequest): Promise<Response> {
	try {
		const { session } = request;

		if (!session) {
			const response: UpdateInfoResponse = {
				code: httpStatus.UNAUTHORIZED,
				success: false,
				error: errorMessages.NOT_AUTHENTICATED,
			};
			return Response.json(response, { status: httpStatus.UNAUTHORIZED });
		}

		const { username, displayName } = request.requestBody as UpdateInfoRequest;

		if (username === undefined && displayName === undefined) {
			const response: UpdateInfoResponse = {
				code: httpStatus.BAD_REQUEST,
				success: false,
				error: "At least one field must be provided (username, displayName)",
			};
			return Response.json(response, { status: httpStatus.BAD_REQUEST });
		}

		const currentUserQuery = `
			SELECT id, username, display_name, email, is_verified, created_at, updated_at
			FROM users WHERE id = ? LIMIT 1
		`;

		const currentUserResult = (await cassandra.execute(currentUserQuery, [
			session.id,
		])) as { rows: UserRow[] };

		if (!currentUserResult?.rows || currentUserResult.rows.length === 0) {
			await sessionManager.invalidateSession(request);

			const response: UpdateInfoResponse = {
				code: httpStatus.NOT_FOUND,
				success: false,
				error: errorMessages.USER_NOT_FOUND,
			};
			return Response.json(response, { status: httpStatus.NOT_FOUND });
		}

		const currentUser = currentUserResult.rows[0];
		if (!currentUser) {
			const response: UpdateInfoResponse = {
				code: httpStatus.NOT_FOUND,
				success: false,
				error: errorMessages.USER_NOT_FOUND,
			};
			return Response.json(response, { status: httpStatus.NOT_FOUND });
		}

		const updates: {
			username?: string;
			displayName?: string | null;
		} = {};

		if (username !== undefined) {
			const usernameValidation = isValidUsername(username);
			if (!usernameValidation.valid || !usernameValidation.username) {
				const response: UpdateInfoResponse = {
					code: httpStatus.BAD_REQUEST,
					success: false,
					error: usernameValidation.error || "Invalid username",
				};
				return Response.json(response, { status: httpStatus.BAD_REQUEST });
			}

			if (usernameValidation.username !== currentUser.username) {
				const existingUsernameQuery =
					"SELECT id FROM users WHERE username = ? LIMIT 1";
				const existingUsernameResult = (await cassandra.execute(
					existingUsernameQuery,
					[usernameValidation.username],
				)) as { rows: Array<{ id: string }> };

				if (
					existingUsernameResult.rows.length > 0 &&
					existingUsernameResult.rows[0]?.id !== session.id
				) {
					const response: UpdateInfoResponse = {
						code: httpStatus.CONFLICT,
						success: false,
						error: errorMessages.USERNAME_ALREADY_EXISTS,
					};
					return Response.json(response, { status: httpStatus.CONFLICT });
				}

				updates.username = usernameValidation.username;
			}
		}

		if (displayName !== undefined) {
			if (displayName === null || displayName.trim() === "") {
				updates.displayName = null;
			} else {
				const displayNameValidation = isValidDisplayName(displayName);
				if (!displayNameValidation.valid) {
					const response: UpdateInfoResponse = {
						code: httpStatus.BAD_REQUEST,
						success: false,
						error: displayNameValidation.error || "Invalid display name",
					};
					return Response.json(response, { status: httpStatus.BAD_REQUEST });
				}
				updates.displayName = displayNameValidation.name || null;
			}
		}

		if (Object.keys(updates).length === 0) {
			const response: UpdateInfoResponse = {
				code: httpStatus.OK,
				success: true,
				message: "No changes required",
				user: {
					id: currentUser.id,
					username: currentUser.username,
					displayName: currentUser.display_name,
					email: currentUser.email,
					isVerified: currentUser.is_verified,
					createdAt: currentUser.created_at.toISOString(),
				},
			};
			return Response.json(response, { status: httpStatus.OK });
		}

		const updateFields: string[] = [];
		const updateValues: unknown[] = [];

		if (updates.username !== undefined) {
			updateFields.push("username = ?");
			updateValues.push(updates.username);
		}

		if (updates.displayName !== undefined) {
			updateFields.push("display_name = ?");
			updateValues.push(updates.displayName);
		}

		updateFields.push("updated_at = ?");
		updateValues.push(new Date());
		updateValues.push(session.id);

		const updateQuery = `
			UPDATE users
			SET ${updateFields.join(", ")}
			WHERE id = ?
		`;

		await cassandra.execute(updateQuery, updateValues);

		const updatedUserResult = (await cassandra.execute(currentUserQuery, [
			session.id,
		])) as { rows: UserRow[] };

		const updatedUser = updatedUserResult.rows[0];
		if (!updatedUser) {
			const response: UpdateInfoResponse = {
				code: httpStatus.INTERNAL_SERVER_ERROR,
				success: false,
				error: "Failed to fetch updated user data",
			};
			return Response.json(response, {
				status: httpStatus.INTERNAL_SERVER_ERROR,
			});
		}

		const userAgent = request.headers.get("User-Agent") || "Unknown";
		const updatedSessionPayload = {
			id: updatedUser.id,
			username: updatedUser.username,
			email: updatedUser.email,
			isVerified: updatedUser.is_verified,
			displayName: updatedUser.display_name,
			createdAt: updatedUser.created_at.toISOString(),
			updatedAt: updatedUser.updated_at.toISOString(),
		};

		const sessionCookie = await sessionManager.updateSession(
			request,
			updatedSessionPayload,
			userAgent,
		);

		const responseUser: UserResponse = {
			id: updatedUser.id,
			username: updatedUser.username,
			displayName: updatedUser.display_name,
			email: updatedUser.email,
			isVerified: updatedUser.is_verified,
			createdAt: updatedUser.created_at.toISOString(),
		};

		const response: UpdateInfoResponse = {
			code: httpStatus.OK,
			success: true,
			message: successMessages.USER_INFO_UPDATED,
			user: responseUser,
		};

		return Response.json(response, {
			status: httpStatus.OK,
			headers: {
				"Set-Cookie": sessionCookie,
			},
		});
	} catch (error) {
		echo.error({
			message: "Error updating user information",
			error,
		});

		const response: UpdateInfoResponse = {
			code: httpStatus.INTERNAL_SERVER_ERROR,
			success: false,
			error: errorMessages.INTERNAL_SERVER_ERROR,
		};
		return Response.json(response, {
			status: httpStatus.INTERNAL_SERVER_ERROR,
		});
	}
}

export { handler, routeDef };
